
/**
 * Klasse zur Speicherung der biometrischen Merkmale eines Gesichts. Die Berechnung der Merkmale erfolgt bei Erstellung 
 * eines Objekts durch den Konstruktor. 
 * 
 * @version 1.0
 */
public class GESICHT
{
    private double[] merkmale;  // Feld fr die berechneten Merkmale des Gesichts
    private String name;        // Names des Gesichts (z.B. Dateiname des Bildes)
    private PERSON person;      // Referenz auf die zu diesem Gesicht gehrende Person

    // Folgende Konstanten sollen die Berechnung der Merkmale erleichtern. 
    private final int LOhrL = 0;
    private final int LOhrU = 2;
    private final int LAuge = 4;
    private final int LAugeL = 6;
    private final int LAugeR = 8;
    private final int ROhrR = 10;
    private final int ROhrU = 12;
    private final int RAuge = 14;
    private final int RAugeL = 16;
    private final int RAugeR = 18;
    private final int LNase = 20;
    private final int RNase = 22;
    private final int MundL = 24;
    private final int MundR = 26;
    private final int MundLO = 28;
    private final int MundRO = 30;
    private final int KopfU = 32;
    private final int KopfO = 34;
    private final int KopfL = 36;
    private final int KopfR = 38;
    private final int NaseL = 40;
    private final int NaseR = 42;

    /**
     * Erstellt ein Objekt der Klasse GESICHT.
     * 
     * @param aName   Name des Bildes, aus dem Koordinaten stammen (z.B. Dateiname)
     * @param aKoords Liste der erfassten Koordinaten des Gesichts
     */
    public GESICHT(PERSON aPerson, String aName, double[] aKoords)
    {
        person = aPerson;
        name = aName;
        merkmale = new double[37];  
        int m = 0;
        // 0-1
        merkmale[m++] = abstand(aKoords[KopfU], aKoords[KopfU+1], aKoords[KopfO], aKoords[KopfO+1]); // Hhe des Gesichts
        merkmale[m++] = abstand(aKoords[KopfL], aKoords[KopfL+1], aKoords[KopfR], aKoords[KopfR+1]); // Breite des Gesichts

        // Augen 2-9
        merkmale[m++] = 0.5 * abstand(aKoords[LAugeL], aKoords[LAugeL+1], aKoords[LAugeR], aKoords[LAugeR+1]); // Breite des Auges
        merkmale[m++] = 0.5 * abstand(aKoords[RAugeL], aKoords[RAugeL+1], aKoords[RAugeR], aKoords[RAugeR+1]); 
        merkmale[m++] = 0.5 * abstand(aKoords[LAugeR], aKoords[LAugeR+1], aKoords[LAuge], aKoords[LAuge+1]); // Pupille-Auge (zur Nase)
        merkmale[m++] = 0.5 * abstand(aKoords[RAugeL], aKoords[RAugeL+1], aKoords[RAuge], aKoords[RAuge+1]); 
        merkmale[m++] = 0.5 * abstand(aKoords[LAugeL], aKoords[LAugeL+1], aKoords[LAuge], aKoords[LAuge+1]); // Pupille-Auge (nach auen)
        merkmale[m++] = 0.5 * abstand(aKoords[RAugeR], aKoords[RAugeR+1], aKoords[RAuge], aKoords[RAuge+1]); 
        merkmale[m++] = abstand(aKoords[LAugeL], aKoords[LAugeL+1], aKoords[KopfL], aKoords[KopfL+1]); // Auge zum linken Kopfrand
        merkmale[m++] = abstand(aKoords[RAugeR], aKoords[RAugeR+1], aKoords[KopfR], aKoords[KopfR+1]); // Auge zum rechten Kopfrand

        // Nase 10-19
        merkmale[m++] = abstand(aKoords[LNase], aKoords[LNase+1], aKoords[RNase], aKoords[RNase+1]); // Nasenbreite
        merkmale[m++] = abstand(aKoords[LNase], aKoords[LNase+1], 0,0); // rechter Nasenflgel zur Nasenmitte
        merkmale[m++] = abstand(aKoords[RNase], aKoords[RNase+1], 0,0); // linker Nasenflgel zur Nasenmitte
        merkmale[m++] = abstand(aKoords[LNase], aKoords[LNase+1], aKoords[NaseL], aKoords[NaseL+1]); // Linke Nasenflanke (gutes Merkmal)
        merkmale[m++] = abstand(aKoords[RNase], aKoords[RNase+1], aKoords[NaseR], aKoords[NaseR+1]); // Rechte Nasenflanke (gutes Merkmal)
        merkmale[m++] = abstand(aKoords[NaseL], aKoords[NaseL+1], aKoords[NaseR], aKoords[NaseR+1]); // Nasenbreite auf Augenhhe
        merkmale[m++] = abstand(aKoords[NaseL], aKoords[NaseL+1], aKoords[LAugeR], aKoords[LAugeR+1]); // Nase-Auge-Abstand (auf Augenhhe)
        merkmale[m++] = abstand(aKoords[NaseR], aKoords[NaseR+1], aKoords[RAugeL], aKoords[RAugeL+1]); 
        merkmale[m++] = abstand(aKoords[LNase], aKoords[LNase+1], aKoords[LAugeL], aKoords[LAugeL+1]); // Nase-Auge-Abstand (diagonal ber Backe) (gutes Merkmal)
        merkmale[m++] = abstand(aKoords[RNase], aKoords[RNase+1], aKoords[RAugeR], aKoords[RAugeR+1]); 

        // Mund 20-27
        merkmale[m++] = abstand(aKoords[MundL], aKoords[MundL+1], aKoords[MundR], aKoords[MundR+1]); // Mundbreite (gutes Merkmal)
        merkmale[m++] = abstand(aKoords[MundL], aKoords[MundL+1], aKoords[MundLO], aKoords[MundLO+1]); // linke Lippe 
        merkmale[m++] = abstand(aKoords[MundLO], aKoords[MundLO+1], aKoords[MundRO], aKoords[MundRO+1]); // mittlere Lippe
        merkmale[m++] = abstand(aKoords[MundRO], aKoords[MundRO+1], aKoords[MundR], aKoords[MundR+1]); // rechte Lippe
        merkmale[m++] = abstand(aKoords[MundLO], aKoords[MundLO+1], aKoords[LNase], aKoords[LNase+1]); // Mund-Nase links
        merkmale[m++] = abstand(aKoords[MundRO], aKoords[MundRO+1], aKoords[RNase], aKoords[RNase+1]); // Mund-Nase rechts
        merkmale[m++] = abstand(aKoords[MundLO], aKoords[MundLO+1], aKoords[KopfU], aKoords[KopfU+1]); // Mund-Kinn
        merkmale[m++] = abstand(aKoords[MundRO], aKoords[MundRO+1], aKoords[KopfU], aKoords[KopfU+1]); // Mund-Kinn

        // Ohren 28-
        merkmale[m++] = abstand(aKoords[LOhrL], aKoords[LOhrL+1], aKoords[ROhrR], aKoords[ROhrR+1]); // Gesichtsbreite ber Ohren (gutes)
        merkmale[m++] = abstand(aKoords[LOhrL], aKoords[LOhrL+1], aKoords[LOhrU], aKoords[LOhrU+1]); // Linke Ohrkante (gutes)
        merkmale[m++] = abstand(aKoords[ROhrR], aKoords[ROhrR+1], aKoords[ROhrU], aKoords[ROhrU+1]); // Rechte Ohrkante (eher schlecht)
        merkmale[m++] = abstand(aKoords[LOhrL], aKoords[LOhrL+1], aKoords[KopfL], aKoords[KopfL+1]); // Linke Ohrbreite (eher schlecht)
        merkmale[m++] = abstand(aKoords[ROhrR], aKoords[ROhrR+1], aKoords[KopfR], aKoords[KopfR+1]); // Rechte Ohrbreite (gut)
        merkmale[m++] = abstand(aKoords[LOhrU], aKoords[LOhrU+1], aKoords[KopfL], aKoords[KopfL+1]); // Linke Ohrhhe (schlecht)
        merkmale[m++] = abstand(aKoords[ROhrU], aKoords[ROhrU+1], aKoords[KopfR], aKoords[KopfR+1]); // Rechte Ohrhhe (schlecht)
        merkmale[m++] = abstand(aKoords[LOhrU], aKoords[LOhrU+1], aKoords[LNase], aKoords[LNase+1]); // Ohr-Nase links (gutes)
        merkmale[m++] = abstand(aKoords[ROhrU], aKoords[ROhrU+1], aKoords[RNase], aKoords[RNase+1]); // Ohr-Nase rechts (sehr gutes)
    }

    /**
     * Liefert den Abstand zwischen zwei Punkten (x1,y1) und (x2,y2). Die Implementierung nutzt zurzeit einen 
     * euklischen Abstand wie er in der Schulegeometrie blich ist.
     * 
     * @param x1 X-Koordinate des ersten Punktes
     * @param y1 Y-Koordinate des ersten Punktes
     * @param x2 X-Koordinate des zweiten Punktes
     * @param y2 Y-Koordinate des zweiten Punktes
     */
    private double abstand(double x1, double y1, double x2, double y2)
    {
        return Math.sqrt(Math.pow(x2-x1,2) + Math.pow(y2-y1,2));        
    }    

    /**
     * Liefert die Anzahl der berechneten Merkmale. Der Rckgabewert ist letztendlich konstant, d.h. er ergibt 
     * sich aus den im Konstruktor berechneten Merkmalen. Um andere Klassen unabhngig nderungen im Konstruktor
     * implementieren zu knnen, ist die Anzahl der Merkmale jedoch ber diese Methode abrufbar.
     */
    public int getMerkmalzahl()
    {
        return merkmale.length;
    }    

    /**
     * Liefert den Wert des i-ten Merkmals. Wird ein ungltiger Index bergeben wird der Wert 0 zurckgeliefert 
     * und eine Fehlermeldung ausgegeben.
     * 
     * @param aIndex Index des gewnschten Merkmals
     */ 
    public double getMerkmal(int aIndex)
    {
        if ((aIndex >= 0) && (aIndex <= merkmale.length))
        {
            return merkmale[aIndex];
        }  
        System.out.println("Es wurde versucht auf das nicht existierende "+aIndex+". Merkmal zuzugreifen.");
        return 0;
    }    

    /**
     * Liefert den bei der Erzeugung der Klasse angegeben Namen des Gesichts zurck (z.B. Dateiname).
     */
    public String getGesichtName()
    {
        return name;
    }    

    /**
     * Liefert die bei der Erzeugung der Klasse angegeben Person, zu der dieses Gesicht gehrt, zurck.
     */
    public PERSON getPerson()
    {
        return person;
    }    
}
